@@ -108,6 +108,9 @@ class Resources(ResourcesBase):
     def auto_create_engine_async(self, name: str | None = None):
         raise Exception("Azure doesn't support auto_create_engine_async")
 
+    def alter_engine_pool(self, size: str | None = None, mins: int | None = None, maxs: int | None = None):
+        raise Exception("Azure doesn't support engine pool alteration")
+
     #--------------------------------------------------
     # Graphs
     #--------------------------------------------------
@@ -286,7 +289,7 @@ class Resources(ResourcesBase):
     # Exec format
     #--------------------------------------------------
 
-    def exec_format(self, database: str, engine: str, raw_code: str, task:m.Task, format:str, inputs: Dict | None = None, readonly: bool = True, nowait_durable=False, headers: Dict | None = None) -> Any: # @FIXME: Better type annotation
+    def exec_format(self, database: str, engine: str, raw_code: str, task:m.Task, format:str, inputs: Dict | None = None, readonly: bool = True, nowait_durable=False, skip_invalid_data=False, headers: Dict | None = None) -> Any: # @FIXME: Better type annotation
         raise Exception("Azure doesn't support alternative formats yet")
 
     def to_model_type(self, model:dsl.Graph, name: str, source:str):
