@@ -116,6 +116,7 @@ class ExportParams:
     code: str
     install_code: str
     dry_run: bool
+    skip_invalid_data: bool
     sources: List[str]
 
 #--------------------------------------------------
@@ -198,6 +199,10 @@ class ResourcesBase(ABC):
     def resume_engine_async(self, name: str) -> dict:
         pass
 
+    @abstractmethod
+    def alter_engine_pool(self, size: str | None = None, mins: int | None = None, maxs: int | None = None):
+        pass
+
     def get_default_engine_name(self) -> str:
         engine = self._get_active_engine() or self.config.get("engine")
         if not engine:
@@ -404,7 +409,7 @@ class ResourcesBase(ABC):
         pass
 
     @abstractmethod
-    def exec_format(self, database: str, engine: str, raw_code: str, task:m.Task, format:str, inputs: Dict | None = None, readonly: bool = True, nowait_durable=False, headers: Dict | None = None) -> Any: # @FIXME: Better type annotation
+    def exec_format(self, database: str, engine: str, raw_code: str, task:m.Task, format:str, inputs: Dict | None = None, readonly: bool = True, nowait_durable=False, skip_invalid_data=False, headers: Dict | None = None) -> Any: # @FIXME: Better type annotation
         pass
 
     @abstractmethod
@@ -736,8 +741,8 @@ class Client():
                 raise e
 
 
-    def _query_format(self, code:str, task:m.Task, end_span, format, readonly=False, inputs: Dict | None = None):
-        results, raw = self.resources.exec_format(self._database, self.get_engine_name(), code, task, readonly=readonly, inputs=inputs, format=format)
+    def _query_format(self, code:str, task:m.Task, end_span, format, readonly=False, skip_invalid_data=False, inputs: Dict | None = None):
+        results, raw = self.resources.exec_format(self._database, self.get_engine_name(), code, task, readonly=readonly, inputs=inputs, format=format, skip_invalid_data=skip_invalid_data)
         errors = []
         if raw:
             dataframe, errors = self.resources.format_results(raw, task)
@@ -747,7 +752,7 @@ class Client():
         # return results if raw_results else dataframe
         return results, raw
 
-    def query(self, task:m.Task, rentrant=False, read_only=False, raw_results=False, inputs: Dict | None = None, format="pandas", tag=None, nowait_durable=None, headers: Dict | None = None, internal=False, abort_on_error=True) -> DataFrame|Any:
+    def query(self, task:m.Task, rentrant=False, read_only=False, raw_results=False, inputs: Dict | None = None, format="pandas", tag=None, nowait_durable=None, headers: Dict | None = None, internal=False, abort_on_error=True, skip_invalid_data = False) -> DataFrame|Any:
         if not self.dry_run and self.use_graph_index:
             self.create_database(isolated=self.isolated, headers=headers)
 
@@ -791,7 +796,7 @@ class Client():
                 results, raw = self._query(code, task, end_span, readonly=read_only, inputs=inputs, nowait_durable=nowait_durable, headers=headers, abort_on_error=abort_on_error)
                 debugging.time("query", time.perf_counter() - start, DataFrame() if raw_results else results, internal=internal, source_map=source_map)
             else:
-                results, raw = self._query_format(code, task, end_span, readonly=read_only, inputs=inputs, format=format)
+                results, raw = self._query_format(code, task, end_span, readonly=read_only, inputs=inputs, format=format, skip_invalid_data=skip_invalid_data)
                 debugging.time("query", time.perf_counter() - start, DataFrame(), source_map=source_map, alt_format_results=results)
 
             self._install_batch.clear_dirty()
@@ -842,7 +847,7 @@ class Client():
             code = self.compiler.compile(task)
             self._install_batch.append("pyrel_batch_0", code, task=task)
 
-    def export_udf(self, name, inputs:List[Tuple[str, m.Var, Any]], outputs, task:m.Task, engine:str|None):
+    def export_udf(self, name, inputs:List[Tuple[str, m.Var, Any]], outputs, task:m.Task, engine:str|None, skip_invalid_data:bool=False):
         if engine is None:
             engine = ""
 
@@ -873,6 +878,7 @@ class Client():
             code=rel_code,
             install_code=installs,
             dry_run=self.dry_run,
+            skip_invalid_data=skip_invalid_data,
             sources=[]
         )
         self.resources.create_export(params)
