@@ -11,7 +11,7 @@ import tomlkit
 import tomlkit.exceptions
 
 from relationalai.clients.hash_util import hash_string
-from relationalai.tools.constants import FIELD_PLACEHOLDER, SNOWFLAKE_AUTHS, SNOWFLAKE_PROFILE_DEFAULTS
+from relationalai.tools.constants import FIELD_PLACEHOLDER, SNOWFLAKE_AUTHS, SNOWFLAKE_DIRECT_ACCESS_AUTHS, SNOWFLAKE_PROFILE_DEFAULTS
 from relationalai.util.constants import DEFAULT_PROFILE_NAME, TOP_LEVEL_PROFILE_NAME
 
 PUBLIC_CONFIG_KEYS = [
@@ -463,6 +463,7 @@ class Config():
                 self.set(k, v)
             if not self.file_path:
                 self.file_path = "__inline__"
+        self._handle_snowflake_fallback_configurations()
 
     def fetch(self, profile:str|None=None):
         from relationalai.environments import runtime_env, TerminalEnvironment
@@ -551,6 +552,17 @@ class Config():
 
     def to_rai_config(self) -> Dict[str, Any]:
         return to_rai_config(self.props)
+    
+    def _handle_snowflake_fallback_configurations(self):
+        if not self.get("platform", "") == "snowflake":
+            return
+        if self.get("use_direct_access", False):
+            if str(self.get("authenticator", "")).lower() not in SNOWFLAKE_DIRECT_ACCESS_AUTHS:
+                # import here to avoid circular imports
+                from relationalai.debugging import warn
+                from relationalai.errors import DirectAccessInvalidAuthWarning
+                warn(DirectAccessInvalidAuthWarning(str(self.get("authenticator", "")), SNOWFLAKE_DIRECT_ACCESS_AUTHS.keys()))
+                self.set("use_direct_access", False)
 
     def _fill_in_with_defaults(self, defaults: Dict[str, Any], **kwargs):
         props = {k: v for k, v in kwargs.items() if k in defaults}
