@@ -1,5 +1,6 @@
 from __future__ import annotations
 import json
+import logging
 import random
 import time
 from typing import Optional, TypeVar
@@ -7,11 +8,24 @@ import urllib.parse
 import re
 from datetime import datetime, timezone
 
-from relationalai.tools.constants import Generation
+from relationalai.clients.config import Config
+from relationalai.tools.constants import SHOW_DEBUG_LOGS, Generation
 import requests
 
 from relationalai.errors import RAIException
 
+# Set up a logger for get_with_retries
+get_with_retries_logger = logging.getLogger(f"{__name__}.get_with_retries")
+get_with_retries_logger.setLevel(logging.INFO)
+
+# Add a console handler only if one doesn't exist
+if not get_with_retries_logger.handlers:
+    handler = logging.StreamHandler()
+    formatter = logging.Formatter('%(name)s:%(levelname)s:%(message)s')
+    handler.setFormatter(formatter)
+    get_with_retries_logger.addHandler(handler)
+    get_with_retries_logger.propagate = False
+
 # replace the values of the URL parameters that start with X-Amz- with XXX
 def scrub_url(url):
     parsed = urllib.parse.urlparse(url)
@@ -109,33 +123,55 @@ def get_with_retries(
     max_retries: int = 3,
     backoff_factor = 2,
     min_backoff_s = 2,
+    config: Config | None = None
 ):
+    show_debug_logs = config.get("show_debug_logs", SHOW_DEBUG_LOGS) if config else SHOW_DEBUG_LOGS
+    prefix = "get_with_retries"
+
+    def _log(msg: str, **fields):
+        if not show_debug_logs:
+            return
+        line = f"[{prefix}] {msg}"
+        if fields:
+            kv = " ".join(f"{k}={v}" for k, v in fields.items())
+            line = f"{line} | {kv}"
+        get_with_retries_logger.info(line)
 
     jitter_s = random.uniform(0.5, 1.5)
+
     def pause(delay_s):
         """ Given the previous delay, sleep for a bit and return the new delay. """
         delay_s *= backoff_factor
-        time.sleep(delay_s + jitter_s)
+        sleep_s = delay_s + jitter_s
+        _log("backing off", sleep_s=round(sleep_s, 3))
+        time.sleep(sleep_s)
         return delay_s
 
     attempt = 0
     delay_s = min_backoff_s
     while True:
         attempt += 1
+        _log("attempting request", attempt=attempt, url=url)
         try:
             res = session.get(url)
+            _log("received response", attempt=attempt, status=res.status_code)
             if 500 <= res.status_code <= 504:
                 if attempt > max_retries:
+                    _log("max retries reached; raising for status", attempt=attempt)
                     res.raise_for_status()
                 else:
                     delay_s = pause(delay_s)
                     continue
             return res
-        except ConnectionResetError as e:
+        # Retry on any error, network issues can manifest in many unexpected ways
+        except Exception as e:
             if attempt > max_retries:
+                _log("max retries reached; re-raising exception", attempt=attempt, err=type(e).__name__, error=str(e))
                 raise e
             else:
+                _log("error occurred; will retry", attempt=attempt, err=type(e).__name__, error=str(e))
                 delay_s = pause(delay_s)
+                continue
 
 class ParseError(Exception):
     def __init__(self, identifier):
