@@ -4,7 +4,7 @@ import contextlib
 import datetime
 import json
 import os
-from typing import Dict, Generator
+from typing import Dict, Generator, Optional
 import logging
 import uuid
 import warnings
@@ -36,7 +36,7 @@ def encode_log_message(obj):
         df = obj.replace({np.nan: None})
         return df.head(20).to_dict(orient="records")
     if isinstance(obj, datetime.datetime):
-        return obj.isoformat() + "Z"
+        return obj.isoformat()
     if isinstance(obj, dict):
         return {k: encode_log_message(v) for k, v in obj.items()}
     if hasattr(obj, "to_dict"):
@@ -153,7 +153,7 @@ def span_end(span):
     msg = {
         "event": "span_end",
         "id": str(span.id),
-        "end_timestamp": span.end_timestamp.isoformat() + "Z",
+        "end_timestamp": span.end_timestamp.isoformat(),
         "elapsed": pytime.perf_counter() - span.precise_start,
         "end_attrs": span.end_attrs,
     }
@@ -207,11 +207,20 @@ class Span:
             "id": str(self.id),
             "parent_id": str(self.parent.id) if self.parent else None,
             "trace_id": str(self.trace_id),
-            "start_timestamp": self.start_timestamp.isoformat() + "Z",
-            "end_timestamp": None if self.end_timestamp is None else (self.end_timestamp.isoformat() + "Z"),
+            "start_timestamp": self.start_timestamp.isoformat(),
+            "end_timestamp": None if self.end_timestamp is None else self.end_timestamp.isoformat(),
             "attrs": self.attrs,
         }
 
+
+def add_current_propagation_headers(headers: Optional[Dict]) -> Dict:
+    prop_headers = gen_current_propagation_headers()
+
+    if headers:
+        prop_headers.update(headers)        
+
+    return prop_headers
+
 def gen_current_propagation_headers():
     return generate_propagation_headers(get_current_span())
 
