@@ -371,7 +371,7 @@ class Context():
     def __init__(self, graph:'Graph', *args, behavior=Behavior.Query, op=None,
                  exec_type=TaskExecType.Rule, dynamic=False, format="default",
                  props=None, engine=None, tag=None, globalize=False, source=None,
-                 attributes=None, read_only=False):
+                 attributes=None, read_only=False, skip_invalid_data=False):
         self._id = next_id()
         self.results = DataFrame()
         self.graph = graph
@@ -389,6 +389,7 @@ class Context():
         self._done = False
         self._source = source
         self._read_only:bool = read_only
+        self._skip_invalid_data:bool = skip_invalid_data
 
         self._validate_attributes(attributes)
         self._attributes = attributes
@@ -1611,9 +1612,9 @@ class Graph:
             self._client.install(f"rule{len(self._executed)}", context._task)
         elif context._exec_type == TaskExecType.Query:
             headers = {QUERY_ATTRIBUTES_HEADER: json.dumps(context._attributes)} if context._attributes else {}
-            context.results = self._client.query(context._task, tag=context._tag, format=context._format, headers=headers, read_only=context._read_only)
+            context.results = self._client.query(context._task, tag=context._tag, format=context._format, headers=headers, read_only=context._read_only, skip_invalid_data=context._skip_invalid_data)
         elif context._exec_type == TaskExecType.Procedure:
-            self._client.export_udf(props["name"], props["inputs"], props["outputs"], context._task, props.get("engine"))
+            self._client.export_udf(props["name"], props["inputs"], props["outputs"], context._task, props.get("engine"), skip_invalid_data=context._skip_invalid_data)
         elif context._exec_type == TaskExecType.Export:
             self._client.export_table(props["name"], props["rel"], props["columns"], context._task, engine=props.get("engine"), refresh=props.get("refresh"))
         self._executed.append(context)
