@@ -11,7 +11,7 @@ from relationalai.early_access.dsl.adapters.orm.model import ORMEntityType, ORMV
     ORMRoleSubsetConstraint, ORMInclusiveRoleConstraint, ORMExclusiveRoleConstraint, ORMEqualityConstraint, \
     ORMFrequencyConstraint, ORMCardinalityConstraint, ORMRange, ORMRoleCardinalityConstraint, \
     ORMValueTypeValueConstraint
-from relationalai.early_access.metamodel.util import ordered_set
+from relationalai.semantics.metamodel.util import ordered_set
 from relationalai.util.graph import topological_sort
 
 
@@ -32,7 +32,7 @@ class ORMParser:
         self._entity_types = self._parse_entity_types()
         self._value_types = self._parse_value_types()
         self._object_types = {**self._value_types, **self._entity_types}
-        self._roles = self._parse_roles()
+        self._roles, self._ignored_roles = self._parse_roles()
         self._fact_type_to_roles = self._parse_fact_type_to_roles()
         self._role_value_constraints = self._parse_role_value_constraints()
         self._internal_uniqueness_constraints, self._external_uniqueness_constraints = self._parse_uniqueness_constraints()
@@ -209,20 +209,23 @@ class ORMParser:
 
     def _parse_roles(self):
         roles = {}
+        ignored_roles = []
         orm_fact_types = self._get_nested(self._ontology, "ormRoot:ORM2", "orm:ORMModel", "orm:Facts", "orm:Fact")
         if orm_fact_types:
             for ft in self._single_object_to_list(orm_fact_types):
-                relationship_name = (ft['@_Name'])
+                relationship_name = ft['@_Name']
+                unary_pattern = ft.get("@UnaryPattern", None)  # unary patterns are not supported
                 orm_roles = self._get_nested(ft, "orm:FactRoles", "orm:Role")
                 if orm_roles:
                     for ro in self._single_object_to_list(orm_roles):
                         role_id = ro["@id"]
                         role_name = ro["@Name"]
-                        role_player = self._get_nested(ro, "orm:RolePlayer", "@ref")
-                        if role_player in self._object_types:
+                        if (unary_pattern is not None and unary_pattern == "Negation") or role_id not in self._role_to_player.keys():
+                            ignored_roles.append(role_id)
+                        else:
                             roles[role_id] = ORMRole(role_id, role_name, relationship_name, self._role_to_player[role_id])
-                        self._parse_role_cardinality_constraint(ro)
-        return roles
+                            self._parse_role_cardinality_constraint(ro)
+        return roles, ignored_roles
 
     def _parse_role_to_player_reference(self, object_type):
         roles = self._get_nested(object_type, "orm:PlayedRoles", "orm:Role")
@@ -238,10 +241,12 @@ class ORMParser:
                 roles = self._get_nested(ft, "orm:FactRoles", "orm:Role")
                 if roles and isinstance(roles, list):
                     for ro in roles:
-                        role_value_constraint = self._get_nested(ro, "orm:ValueRestriction", "orm:RoleValueConstraint")
-                        if role_value_constraint:
-                            range_values = self._parse_constraint_range(role_value_constraint)
-                            role_value_constraints[ro["@id"]] = ORMRoleValueConstraint(self._roles[ro["@id"]], range_values)
+                        role_id = ro.get("@id")
+                        if role_id not in self._ignored_roles:
+                            role_value_constraint = self._get_nested(ro, "orm:ValueRestriction", "orm:RoleValueConstraint")
+                            if role_value_constraint:
+                                range_values = self._parse_constraint_range(role_value_constraint)
+                                role_value_constraints[role_id] = ORMRoleValueConstraint(self._roles[role_id], range_values)
         return role_value_constraints
 
     def _parse_constraint_range(self, orm_constraint_object):
@@ -263,7 +268,7 @@ class ORMParser:
     def _parse_inclusive_role_constraints(self):
         constraints = []
         for ec in self._inclusion_constraints.values():
-            if ec.roles and set(ec.roles).issubset(self._roles):
+            if ec.roles and set(ec.roles).issubset(self._roles) and all(ro not in self._ignored_roles for ro in ec.roles):
                 constraints.append(ORMInclusiveRoleConstraint(ec.id, ec.roles))
         return constraints
 
@@ -271,7 +276,7 @@ class ORMParser:
         constraints = []
         for ec in self._exclusion_constraints.values():
             flat_roles = [role for role_list in ec.roles for role in role_list]
-            if ec.roles and set(flat_roles).issubset(self._roles):
+            if ec.roles and set(flat_roles).issubset(self._roles) and all(ro not in self._ignored_roles for ro in flat_roles):
                 constraints.append(ORMExclusiveRoleConstraint(ec.id, ec.roles))
         return constraints
 
@@ -311,9 +316,10 @@ class ORMParser:
                 identifies = None
                 if pid is not None and pid["@ref"] in self._entity_types:
                     identifies = pid["@ref"]
-                constraint = ORMUniquenessConstraint(uc_id, self._parse_role_sequence(uc), identifies)
-                target = internal if uc.get("@IsInternal") == "true" else external
-                target[uc_id] = constraint
+                roles = self._parse_role_sequence(uc)
+                if roles:
+                    target = internal if uc.get("@IsInternal") == "true" else external
+                    target[uc_id] = ORMUniquenessConstraint(uc_id, roles, identifies)
         return internal, external
 
     def _process_internal_uniqueness_constraints(self):
@@ -351,8 +357,9 @@ class ORMParser:
             for mc in self._single_object_to_list(orm_mcs):
                 mc_id = mc["@id"]
                 if mc.get("@IsSimple", None) is not None:
-                    constraint = ORMMandatoryConstraint(mc_id, self._parse_role_sequence(mc))
-                    mandatory_constraints[mc_id] = constraint
+                    roles = self._parse_role_sequence(mc)
+                    if roles:
+                        mandatory_constraints[mc_id] = ORMMandatoryConstraint(mc_id, roles)
         return mandatory_constraints
 
     def _parse_value_comparison_constraints(self):
@@ -364,8 +371,9 @@ class ORMParser:
                 vcc_id = vcc["@id"]
                 vcc_op = vcc["@Operator"]
                 roles = self._parse_role_sequence(vcc)
-                constraint = ORMValueComparisonConstraint(vcc_id, ORMValueComparisonOperator(vcc_op), roles)
-                value_comparison_constraints[vcc_id] = constraint
+                if roles:
+                    constraint = ORMValueComparisonConstraint(vcc_id, ORMValueComparisonOperator(vcc_op), roles)
+                    value_comparison_constraints[vcc_id] = constraint
         return value_comparison_constraints
 
     def _parse_role_subset_constraints(self):
@@ -382,7 +390,8 @@ class ORMParser:
             for role_sequence_constraint in self._single_object_to_list(orm_constraints):
                 id = role_sequence_constraint["@id"]
                 roles = self._parse_role_sequences(role_sequence_constraint)
-                role_sequence_constraints[id] = constraint_type(id, roles)
+                if roles:
+                    role_sequence_constraints[id] = constraint_type(id, roles)
         return role_sequence_constraints
 
     def _parse_inclusion_constraints(self):
@@ -395,8 +404,9 @@ class ORMParser:
                 mc_name = mc["@Name"]
                 if mc_name.startswith("InclusiveOrConstraint"):
                     exclusive = mc.get("orm:ExclusiveOrExclusionConstraint", None) is not None
-                    constraint = ORMInclusionConstraint(mc_id, self._parse_role_sequence(mc), exclusive)
-                    inclusion_constraints[mc_id] = constraint
+                    roles = self._parse_role_sequence(mc)
+                    if roles:
+                        inclusion_constraints[mc_id] = ORMInclusionConstraint(mc_id, roles, exclusive)
         return inclusion_constraints
 
     def _parse_exclusion_constraints(self):
@@ -407,8 +417,9 @@ class ORMParser:
             for ec in self._single_object_to_list(orm_mcs):
                 ec_id = ec["@id"]
                 inclusive = ec.get("orm:ExclusiveOrMandatoryConstraint", None) is not None
-                constraint = ORMExclusionConstraint(ec_id, self._parse_role_sequences(ec), inclusive)
-                exclusion_constraints[ec_id] = constraint
+                role_sequences = self._parse_role_sequences(ec)
+                if role_sequences:
+                    exclusion_constraints[ec_id] = ORMExclusionConstraint(ec_id, role_sequences, inclusive)
         return exclusion_constraints
 
     def _parse_ring_constraints(self):
@@ -419,9 +430,9 @@ class ORMParser:
             for rc in self._single_object_to_list(orm_rcs):
                 rc_id = rc["@id"]
                 rc_type = rc["@Type"]
-                constraint = ORMRingConstraint(rc_id, self._parse_role_sequence(rc),
-                                               self._process_ring_constraint(rc_type))
-                ring_constraints[rc_id] = constraint
+                roles = self._parse_role_sequence(rc)
+                if roles:
+                    ring_constraints[rc_id] = ORMRingConstraint(rc_id, roles, self._process_ring_constraint(rc_type))
         return ring_constraints
 
     def _parse_frequency_constraints(self):
@@ -433,8 +444,9 @@ class ORMParser:
                 fc_id = fc["@id"]
                 min_freq = fc["@MinFrequency"]
                 max_freq = fc["@MaxFrequency"]
-                constraint = ORMFrequencyConstraint(fc_id, min_freq, max_freq, self._parse_role_sequence(fc))
-                frequency_constraints[fc_id] = constraint
+                roles = self._parse_role_sequence(fc)
+                if roles:
+                    frequency_constraints[fc_id] = ORMFrequencyConstraint(fc_id, min_freq, max_freq, roles)
         return frequency_constraints
 
     def _parse_fact_types_reading_orders(self):
@@ -569,17 +581,26 @@ class ORMParser:
                 if roles:
                     for ro in self._single_object_to_list(roles):
                         role_id = ro["@ref"]
-                        role_sequence.append(role_id)
+                        # Ignore the entire constraint if it involves an ignored role
+                        if role_id in self._ignored_roles:
+                            return None
+                        else:
+                            role_sequence.append(role_id)
                     result.append(role_sequence)
         return result
 
     def _parse_role_sequence(self, orm_constraint):
-        role_sequence = []
         roles = self._get_nested(orm_constraint, "orm:RoleSequence", "orm:Role")
+        role_sequence = []
         if roles:
             for ro in self._single_object_to_list(roles):
                 role_id = ro["@ref"]
-                role_sequence.append(role_id)
+                # Ignore the entire constraint if it involves an ignored role
+                if role_id in self._ignored_roles:
+                    role_sequence = None
+                    break
+                else:
+                    role_sequence.append(role_id)
         return role_sequence
 
     def _sort_subtype_facts(self):
@@ -612,4 +633,4 @@ class ORMParser:
 
     @staticmethod
     def _single_object_to_list(parsed_object):
-        return parsed_object if isinstance(parsed_object, list) else [parsed_object]
\ No newline at end of file
+        return parsed_object if isinstance(parsed_object, list) else [parsed_object]
