@@ -5,15 +5,17 @@ from enum import Enum
 from typing import Optional, TypeVar, Generic, Tuple, Callable, Sequence
 from typing import Union
 
-import relationalai.early_access.builder as qb
-from relationalai.early_access.builder import std
-from relationalai.early_access.builder.builder import Not, Ref
+import relationalai.semantics as qb
+from relationalai.semantics import std
+from relationalai.semantics.internal import internal as b
+from relationalai.semantics.internal.internal import Not
 from relationalai.early_access.dsl.orm.relationships import Relationship, Role
 from relationalai.early_access.dsl.orm.types import Concept
 from relationalai.early_access.dsl.snow.common import ColumnRef, ForeignKey
 
 _PrimitiveType = Union[int, float, str, bool, datetime.date, datetime.datetime, Enum]
-FilterBy = Union[_PrimitiveType, Not, qb.Expression, tuple[qb.Expression]]
+_CompositeFilterBy = Union[Not, qb.Expression, qb.Fragment, b.Producer]
+FilterBy = Union[_PrimitiveType, _CompositeFilterBy, tuple[_CompositeFilterBy, ...]]
 
 _BuiltInQbTransformer = Callable  # A callable that is a built-in QB transformer, such as string_trim, lowercase, etc.
 _Transformer = Union[_BuiltInQbTransformer, qb.Relationship]
@@ -36,24 +38,6 @@ def _get_transform_types(obj: _Transformer):
     raise ValueError(f"Cannot infer input/output types from {obj}. Make sure to use a valid built-in transformer or"
                      f" a binary relationship")
 
-def _validate_transform_with(transform_with: Optional[TransformWith], input_type: Optional[qb.Concept]=None):
-    if not transform_with:
-        return
-
-    elements = (transform_with,) if not isinstance(transform_with, tuple) else transform_with
-
-    # Check the chain of types
-    prev_output_type = input_type
-    for idx, elem in enumerate(elements):
-        if not isinstance(elem, (qb.Relationship , Callable)):
-            raise ValueError(f"Expected a built-in transformer or a binary relationship, got {type(elem)}")
-        input_type, output_type = _get_transform_types(elem)
-        if prev_output_type is not None:
-            if input_type is not prev_output_type:
-                raise ValueError(f"Type mismatch during transformation: output of #{idx-1} ({prev_output_type}) !="
-                                 f" input of #{idx} ({input_type})")
-        prev_output_type = output_type
-
 # defining symbol in codegen to not expose this to the user
 Symbol = Concept.builtins["Symbol"]
 
@@ -62,6 +46,11 @@ class BindableTable(ABC):
     """
     A class representing a bindable table.
     """
+    _ref: b.Ref
+    _row_field: b.Field
+
+    def _to_keys(self) -> list[b.Ref]:
+        return [self._ref]
 
     @abstractmethod
     def key_type(self) -> qb.Concept:
@@ -74,6 +63,9 @@ class BindableTable(ABC):
 
 class BindableAttribute(ABC):
 
+    def _to_keys(self) -> list[b.Ref]:
+        return self.table._to_keys()
+
     @property
     @abstractmethod
     def table(self) -> BindableTable:
@@ -96,18 +88,6 @@ class BindableAttribute(ABC):
     def column_ref(self) -> ColumnRef:
         pass
 
-    @abstractmethod
-    def decimal_scale(self) -> Optional[int]:
-        pass
-
-    @abstractmethod
-    def decimal_size(self) -> Optional[int]:
-        pass
-
-    @abstractmethod
-    def row_ref(self) -> Ref:
-        pass
-
 
 @dataclasses.dataclass(frozen=True)
 class Binding:
@@ -234,23 +214,17 @@ class SubtypeConceptBinding(AbstractConceptBinding):
         return f'SubtypeConceptBinding[{self.column.table.physical_name()}:{self.column.physical_name()} -> {self.entity_type.name()}]'
 
 
-class BindableColumn(BindableAttribute, qb.Relationship, ABC):
+class BindableColumn(BindableAttribute, ABC):
     _source: BindableTable
     _references_column: Optional[ColumnRef]
 
     def __init__(self, col_name: str, col_type: qb.Concept, source: BindableTable, model):
-        madlib, short_name = self._construct_name(col_name, col_type, source)
-        qb.Relationship.__init__(self, madlib, model=model.qb_model(), short_name=short_name)
         self._col_name = col_name
         self._type = col_type
         self._source = source
         self._orm_model = model
         self._references_column = None
 
-    @abstractmethod
-    def _construct_name(self, col_name: str, col_type: qb.Concept, source: BindableTable) -> Tuple[str, str]:
-        pass
-
     def identifies(self, entity_type: Concept, filter_by: Optional[FilterBy] = None,
                    transform_with: Optional[TransformWith] = None):
         """
@@ -262,7 +236,7 @@ class BindableColumn(BindableAttribute, qb.Relationship, ABC):
         """
         if isinstance(entity_type, Enum):
             raise ValueError(f'Cannot bind to Enum {entity_type}, use `references` instead')
-        _validate_transform_with(transform_with, input_type=self._type)
+        filter_by = self._handle_primitive_filter(filter_by)
         binding = IdentifierConceptBinding(column=self, entity_type=entity_type, filter_by=filter_by,
                                            transform_with=transform_with)
         self._orm_model.binding(binding)
@@ -281,7 +255,7 @@ class BindableColumn(BindableAttribute, qb.Relationship, ABC):
             {source}.TRACKING_NUMER.identifies(TrackingNumber) # constructs instances of the TrackingNumber entity type
             {source}.RETURN_TRACKING_NUMBER.references(TrackingNumber) # looks up existing instances of the TrackingNumber entity type
         """
-        _validate_transform_with(transform_with, input_type=self._type)
+        filter_by = self._handle_primitive_filter(filter_by)
         binding = ReferentConceptBinding(column=self, entity_type=entity_type, filter_by=filter_by,
                                          transform_with=transform_with)
         self._orm_model.binding(binding)
@@ -310,7 +284,7 @@ class BindableColumn(BindableAttribute, qb.Relationship, ABC):
                 {source}.CORPORATE_ACTION_TYPE == "MERGER"
             ))
         """
-        _validate_transform_with(transform_with, input_type=self._type)
+        filter_by = self._handle_primitive_filter(filter_by)
         binding = SubtypeConceptBinding(column=self, entity_type=sub_type, filter_by=filter_by,
                                         transform_with=transform_with)
         self._orm_model.binding(binding)
@@ -342,10 +316,19 @@ class BindableColumn(BindableAttribute, qb.Relationship, ABC):
         else:
             raise Exception(
                 f'Expected ORM Relationship or Role, got {type(elm)} - QB Relationships cannot be used in bindings')
-        _validate_transform_with(transform_with, input_type=self._type)
+        filter_by = self._handle_primitive_filter(filter_by)
         binding = RoleBinding(role=role, column=self, filter_by=filter_by, transform_with=transform_with)
         self._orm_model.binding(binding)
 
+    def _handle_primitive_filter(self, filter_by: Optional[FilterBy]) -> Optional[FilterBy]:
+        if isinstance(filter_by, _PrimitiveType):
+            eq = b.Relationship.builtins["="]
+            return b.Expression(eq, self, filter_by)
+        elif isinstance(filter_by, Tuple):
+            return b.where(*filter_by)
+        else:
+            return filter_by
+
     @property
     def table(self):
         return self._source
@@ -372,9 +355,8 @@ T = TypeVar("T", bound=BindableColumn)
 class AbstractBindableTable(BindableTable, Generic[T], ABC):
     _foreign_keys: set[ForeignKey]
 
-    def __init__(self, name: str, model, foreign_keys: set[ForeignKey]):
-        super().__init__()
-        self._table = name
+    def __init__(self, fqn: str, model, foreign_keys: set[ForeignKey]):
+        self._fqn = fqn
         self._model = model
         self._cols: dict[str, T] = dict()
         self._foreign_keys = foreign_keys
