@@ -1,13 +1,16 @@
-import relationalai.early_access.builder as qb
+import relationalai.semantics as qb
 
 
-def reference_entity(concept: qb.Concept, *args):
+def reference_entity(concept: qb.Concept, *args, check_population: bool=True):
     kwargs = entity_kwargs(concept, *args, shallow=True)
-    return concept.new_identity(**kwargs)
+    if check_population:
+        return concept.filter_by(**kwargs)
+    else:
+        return concept.to_identity(**kwargs)
 
 def construct_entity(concept: qb.Concept, *args):
     kwargs = entity_kwargs(concept, *args)
-    return concept.new(**kwargs)
+    return concept.to_identity(**kwargs)
 
 def entity_kwargs(concept: qb.Concept, *args, shallow=False):
     ref_scheme = concept._ref_scheme(shallow=shallow)
