@@ -1,22 +1,18 @@
 from collections import OrderedDict
 from itertools import product
 from types import MappingProxyType
-from typing import Union, Optional, Sequence
+from typing import Union, Sequence
 
-from relationalai.early_access.builder import where
+import relationalai.semantics as qb
+from relationalai.semantics import where
 from relationalai.early_access.dsl.bindings.common import Binding, RoleBinding, IdentifierConceptBinding, BindableTable, \
     BindableAttribute, SubtypeConceptBinding, ReferentConceptBinding
 from relationalai.early_access.dsl.codegen.binder import Binder
 from relationalai.early_access.dsl.codegen.common import BoundIdentifierConstraint, BoundRelationship
-from relationalai.early_access.dsl.codegen.relations import ValueMap, DecimalValueConverter, \
-    CompositeEntityMap, EntitySubtypeMap, UnionEntityMap, AbstractEntityMap, SimpleConstructorEntityMap, \
-    ReferentEntityMap, ValueConverter, _filtering_view, InlineValueMap, Map, MaterializedEntityMap, \
-    AbstractInlineEntityMap
-from relationalai.early_access.metamodel.util import OrderedSet
-from relationalai.early_access.rel.rel_utils import DECIMAL64_SCALE, DECIMAL128_SCALE
-
-import relationalai.early_access.builder as qb
-
+from relationalai.early_access.dsl.codegen.relations import ValueMap, CompositeEntityMap, EntitySubtypeMap, \
+    UnionEntityMap, AbstractEntityMap, SimpleConstructorEntityMap, ReferentEntityMap, InlineValueMap, \
+    Map, MaterializedEntityMap, AbstractInlineEntityMap
+from relationalai.semantics.metamodel.util import OrderedSet
 
 DEFAULT_WEAVER_CONFIG = MappingProxyType({
     'inline_value_maps': False,
@@ -33,7 +29,6 @@ class Weaver:
         self._binder = Binder(self._model)
 
         # Lookup dictionaries for generated relations
-        self._decimal_value_converters = OrderedDict()
         self._binding_to_value_map: dict[Binding, Union[ValueMap, InlineValueMap]] = OrderedDict()
         self._ctor_binding_to_entity_map:\
             dict[IdentifierConceptBinding, Union[AbstractInlineEntityMap, MaterializedEntityMap]] = OrderedDict()
@@ -58,11 +53,10 @@ class Weaver:
 
     def _generate_value_map(self, binding: Binding):
         role = self._binder.lookup_binding_role(binding)
-        converter = self._get_value_converter(binding.column)
         if self._cfg_should_inline_value_maps():
-            value_map = InlineValueMap(self._model, binding, role, converter)
+            value_map = InlineValueMap(self._model, binding, role)
         else:
-            value_map = ValueMap(self._model, binding, role, converter)
+            value_map = ValueMap(self._model, binding, role)
         self._binding_to_value_map[binding] = value_map
 
     def _look_up_value_map(self, binding: Binding):
@@ -71,30 +65,6 @@ class Weaver:
         """
         return self._binding_to_value_map[binding]
 
-    def _get_value_converter(self, column) -> Optional[ValueConverter]:
-        column_type = column.type()
-        if column_type is qb.Decimal64 or column_type is qb.Decimal128:
-            size = column.decimal_size()
-            scale = column.decimal_scale()
-            if size is None or scale is None:
-                raise ValueError(f"Decimal column {column} has no size or scale")
-            is_default_decimal = (
-                (column_type is qb.Decimal64 and size == 64 and scale == DECIMAL64_SCALE) or
-                (column_type is qb.Decimal128 and size == 128 and scale == DECIMAL128_SCALE)
-            )
-            if is_default_decimal:
-                return None
-            return self._decimal_value_converter(size, scale, column_type)
-        return None
-
-    def _decimal_value_converter(self, size: int, scale: int, type_to: qb.Concept) -> DecimalValueConverter:
-        key = hash((size, scale, type_to))
-        converter = self._decimal_value_converters.get(key)
-        if converter is None:
-            converter = DecimalValueConverter(self._model, size, scale, type_to)
-            self._decimal_value_converters[key] = converter
-        return converter
-
     def _generate_entity_maps(self):
         self._generate_simple_ctor_entity_maps()
         self._generate_referent_entity_maps()
@@ -242,7 +212,7 @@ class Weaver:
         elif self._reasoner.is_exclusive_entity_type(concept):  # exclusive entity type
             entity_map_candidates = self._supertype_to_entity_map.values()
             # note: for now, we look up by table, though needs fixing
-            return self._find_matching_entity_maps(entity_map_candidates, concept, column.table)
+            return self._find_matching_entity_maps(entity_map_candidates, concept, column.table, strict_match=True)
         else:  # entity subtype
             entity_map_candidates = self._subtype_binding_to_entity_map.values()
             return self._find_matching_entity_maps(entity_map_candidates, concept, column)
@@ -260,31 +230,40 @@ class Weaver:
             return self._find_matching_entity_maps(entity_map_candidates, concept, source)
         elif self._reasoner.is_exclusive_entity_type(concept):  # exclusive entity type
             entity_map_candidates = self._supertype_to_entity_map.values()
-            return self._find_matching_entity_maps(entity_map_candidates, concept, source)
+            return self._find_matching_entity_maps(entity_map_candidates, concept, source, strict_match=True)
         else:  # entity subtype
             entity_map_candidates = (list(self._subtype_binding_to_entity_map.values()) +
                                      list(self._filtering_subtype_binding_to_entity_map.values()))
             return self._find_matching_entity_maps(entity_map_candidates, concept, source)
-        # TODO : revise below, should there be a uniqueness check?
-        # if len(role_maps) > 1:
-        #     raise ValueError(f'Cannot generate semantic predicate for {relationship},'
-        #                      ' cannot find a unique entity map for an unbound role')
+        # TODO : revise the above logic and see if uniqueness check is needed for anythine else
 
     @staticmethod
     def _find_matching_entity_maps(entity_maps, concept: qb.Concept,
-                                   source_or_col: Union[BindableTable, BindableAttribute]):
-        matching_entity_maps = OrderedSet()
+                                   source_or_col: Union[BindableTable, BindableAttribute],
+                                   strict_match: bool=False):
+        strictly_matching = OrderedSet()
+        loosely_matching = OrderedSet()
         for entity_map in entity_maps:
-            if isinstance(source_or_col, BindableTable):
-                if entity_map.table() == source_or_col and concept._isa(entity_map.value_player()):
-                    matching_entity_maps.add(entity_map)
-            elif isinstance(source_or_col, BindableAttribute):
-                if entity_map.column() is source_or_col and concept._isa(entity_map.value_player()):
-                    matching_entity_maps.add(entity_map)
+            value_player = entity_map.value_player()
+            if Weaver._matches_source(entity_map, source_or_col):
+                if concept is value_player:
+                    strictly_matching.add(entity_map)
+                elif concept._isa(value_player):
+                    loosely_matching.add(entity_map)
+        matching_entity_maps = strictly_matching if bool(strictly_matching) or strict_match else loosely_matching
         if not matching_entity_maps:
             raise ValueError(f'No entity map found for concept {concept} and {source_or_col}')
         return matching_entity_maps
 
+    @staticmethod
+    def _matches_source(entity_map, source_or_col: Union[BindableTable, BindableAttribute]):
+        if isinstance(source_or_col, BindableTable):
+            return entity_map.table() == source_or_col
+        elif isinstance(source_or_col, BindableAttribute):
+            return entity_map.column() is source_or_col
+        else:
+            return False
+
     def _generate_subtype_entity_maps(self):
         for binding in self._binder.subtype_ctor_bindings():
             self._generate_subtype_entity_map(binding)
@@ -365,11 +344,10 @@ class Weaver:
             raise ValueError(
                 f'Cannot generate semantic predicate for {relationship}, no role maps found')
         for entity_map in entity_maps:
-            row_ref = entity_map.row_ref()
-            entity_ref, subformula_atoms = entity_map.formula(row_ref)
+            entity_ref, subformula_atoms = entity_map.formula()
             where(
                 *subformula_atoms,
-                _filtering_view(row_ref, binding.filter_by)
+                where(binding.filter_by)
             ).define(relationship(entity_ref))
 
     def _generate_nary_predicate_rule(self, bound_relationship: BoundRelationship):
@@ -381,9 +359,10 @@ class Weaver:
         # the respective EntityBinding.
         #=
         role_to_role_maps = OrderedDict()
+        role_map_casts_to = OrderedDict()
         for role in roles:
-            concept = role.player()
-            if concept._is_primitive():
+            role_concept = role.player()
+            if role_concept._is_primitive():
                 # TODO: [SB] I suppose we can do bound_relationship.role_bindings[role] here?
                 role_bindings = self._binder.lookup_role_bindings(role, bound_relationship.table)
                 role_maps = [self._binding_to_value_map[binding] for binding in role_bindings]
@@ -394,6 +373,10 @@ class Weaver:
                     role_bindings = self._binder.lookup_role_bindings(role, bound_relationship.table)
                     role_maps = [entity_map for binding in role_bindings
                                  for entity_map in self._lookup_ref_entity_maps_by_column(role.player(), binding.column)]
+                    for role_map in role_maps:
+                        role_map_concept = role_map.value_player()
+                        if role_map_concept is not role_concept and self._reasoner.in_subtype_closure(role_concept, role_map_concept):
+                            role_map_casts_to[role_map] = role_concept
                 else:
                     # lookup inferred entity map (note: must be unique?)
                     role_maps = [entity_map
@@ -406,11 +389,14 @@ class Weaver:
         role_map_combinations = [list(combination) for combination in product(*role_to_role_maps.values())]  # pyright: ignore[reportCallIssue]
         for role_maps in role_map_combinations:
             assert len(role_maps) > 0, "Must have at least one role map to derive a relationship"
-            row_ref = role_maps[0].row_ref()  # take the row handle from the first role map arbitrarily
-            var_refs, subformulas = zip(*(rm.formula(row_ref) for rm in role_maps))
-            where(
-                *(atom for subformula in subformulas for atom in subformula)
-            ).define(relationship(*var_refs))
+            casted_var_refs, atoms = [], []
+            for role_map in role_maps:
+                var_ref, subformula = role_map.formula()
+                if role_map in role_map_casts_to:
+                    var_ref = role_map_casts_to[role_map](var_ref)
+                casted_var_refs.append(var_ref)
+                atoms.extend(subformula)
+            where(*atoms).define(relationship(*casted_var_refs))
 
     def _try_materialize_entity_map(self, entity_map: AbstractInlineEntityMap) -> Union[AbstractInlineEntityMap, MaterializedEntityMap]:
         if self._cfg_should_inline_entity_maps():
