@@ -6,9 +6,9 @@ from relationalai.early_access.dsl import Relation, AssertedRelation
 from relationalai.early_access.dsl.core.relations import ExportRelation
 from relationalai.early_access.dsl.ontologies.export import Export
 from relationalai.early_access.dsl.ontologies.raw_source import RawSource
-from relationalai.early_access.metamodel import ir, builtins, types, factory as f
-from relationalai.early_access.metamodel.types import Any, Symbol
-from relationalai.early_access.metamodel.util import OrderedSet, ordered_set
+from relationalai.semantics.metamodel import ir, builtins, types, factory as f
+from relationalai.semantics.metamodel.types import Any, Symbol
+from relationalai.semantics.metamodel.util import OrderedSet, ordered_set
 
 from relationalai.early_access.dsl.core.constraints.predicate.atomic import Atom
 from relationalai.early_access.dsl.core.constraints.scalar import ScalarConstraint
@@ -62,7 +62,7 @@ class Compiler:
                 Hash: f.scalar_type("UInt128"),
                 Boolean: types.Bool,
                 Integer: types.Int128,
-                Decimal: types.Decimal128,
+                Decimal: types.Decimal,
                 Float: types.Number,
                 Double: types.Number,
                 String: types.String,
@@ -315,4 +315,4 @@ class Compiler:
                 for column in export.columns
                 for rule in column._rules
             ])
-        )
\ No newline at end of file
+        )
