@@ -19,7 +19,7 @@ from relationalai.early_access.dsl.ontologies.subtyping import SubtypeArrow, Inc
 from relationalai.early_access.dsl.types.concepts import Concept
 from relationalai.early_access.dsl.types.entities import EntityType
 from relationalai.early_access.dsl.types.values import ValueType, ValueSubtype
-from relationalai.early_access.metamodel.util import OrderedSet
+from relationalai.semantics.metamodel.util import OrderedSet
 
 
 class Model:
