@@ -54,6 +54,9 @@ class RoleConstraint(Constraint):
     def _unique_name(self) -> str:
         return self._name
 
+    def _discharged(self) -> bool:
+        return False
+
 class RoleSequenceConstraint(Constraint):
     def role_sequences(self):
         return self._role_sequences
@@ -310,6 +313,32 @@ class RoleValueConstraint(RoleConstraint, Generic[T]):
     def __repr__(self):
         return f'RoleValueConstraint({self._roles[0]._guid()}, values={self._values})'
 
+    def _discharged(self) -> bool:
+        # TODO: Implement logic to determine if the constraint is discharged when a unified reasoner is available
+        # See RAI-40577
+        return False
+
+
+class RoleCardinalityConstraint(RoleConstraint):
+    def __init__(self, role: Role, values: Sequence[Union[int, Range[int]]]):
+        if len(role._relationship._fields) > 1:
+            raise ValueError("Role cardinality constraints can only be applied to unary relationships roles.")
+        super().__init__([role])
+        self._values = values
+
+    def values(self):
+        return self._values
+
+    def _validate(self, ontology) -> bool:
+        # Implement role cardinality validation logic here
+        return True
+
+    def _desugar(self):
+        # todo: Implement QB representation
+        pass
+
+    def __repr__(self):
+        return f'RoleCardinalityConstraint({self._roles[0]._guid()}, values={self._values})'
 
 class CardinalityConstraint(ConceptConstraint, Generic[T]):
     def __init__(self, concept: Concept, values: Sequence[Union[int, Range[int]]]):
@@ -332,6 +361,26 @@ class CardinalityConstraint(ConceptConstraint, Generic[T]):
         return f'CardinalityConstraint({self._concept._name}, values={self._values})'
 
 
+class ValueConstraint(ConceptConstraint, Generic[T]):
+    def __init__(self, concept: Concept, values: Sequence[Union[T, Range[T]]]):
+        super().__init__(concept)
+        self._values = values
+
+    def values(self):
+        return self._values
+
+    def _validate(self, ontology) -> bool:
+        # TODO
+        # Implement value type constraint validation logic here
+        return True
+
+    def _desugar(self):
+        # todo: Implement QB representation
+        pass
+
+    def __repr__(self):
+        return f'ValueConstraint({self._concept._name}, values={self._values})'
+
 class SubtypeConstraint(Constraint):
     _concepts: dict[str, Concept]
 
