@@ -2,8 +2,8 @@ from itertools import chain
 
 from relationalai.early_access.dsl.orm.measures.initializer import get_model, get_reasoner
 from relationalai.early_access.dsl.orm.models import Relationship, Role
-from relationalai.early_access.metamodel.util import ordered_set
-import relationalai.early_access.builder.builder as qb
+from relationalai.semantics.metamodel.util import ordered_set
+import relationalai.semantics as qb
 
 from relationalai.early_access.dsl.orm.measures.dimensions import Dimension, Splicer
 from relationalai.early_access.dsl.orm.measures.role_exprs import ComposeRoleExpr, NilRoleExpr, RoleExpr, SimpleRoleExpr, UnionRoleExpr
