@@ -1,7 +1,7 @@
 from typing import Any
 
-from relationalai.early_access.builder import annotations, Relationship as QBRelationship
-
+from relationalai.semantics import Relationship as QBRelationship
+from relationalai.semantics.internal import annotations
 
 class AssertedRelation(QBRelationship):
     """
