@@ -8,6 +8,8 @@ from .jupyter import JupyterEnvironment
 from .hex import HexEnvironment
 from .colab import ColabEnvironment
 from .snowbook import SnowbookEnvironment
+from .ci import CIEnvironment
+from .terminal import TerminalEnvironment
 
 SessionEnvironment = SessionEnvironment # re-export
 NotebookRuntimeEnvironment = NotebookRuntimeEnvironment # re-export
@@ -19,6 +21,8 @@ ENVIRONMENTS: list[Type[RuntimeEnvironment]] = [
     HexEnvironment,
     JupyterEnvironment,
     IPythonEnvironment,
+    CIEnvironment,
+    TerminalEnvironment,
     GenericEnvironment
 ]
 
