@@ -3,6 +3,7 @@ import sys
 from types import TracebackType
 import ast
 import io
+import json
 import re
 import textwrap
 from typing import Any, List, Literal, NamedTuple, Tuple
@@ -681,6 +682,62 @@ class ModelWarning(RAIWarning):
         marked = mark_source(self.source, self.source.line, self.source.line, indent=2) if self.source else ""
         return "\n\n".join([self.message or "", "  " + marked, row_str])
 
+class RowsDroppedFromTargetTableWarning(RAIWarning):
+    def __init__(self, rejected_rows: list[dict], rejected_rows_count: int, col_names_map: dict):
+        """
+        Warning raised when loading data into a table (exec_into_table API) and some rows are rejected due to erroneous data.
+        """
+        self.parsing_errors = self.replace_column_names(rejected_rows, col_names_map)
+        self.name = "Rejected rows"
+        self.content = self.format_message(rejected_rows, rejected_rows_count, self.parsing_errors)
+        self.rejected_rows = rejected_rows
+        self.rejected_rows_count = rejected_rows_count
+        self.col_names_map = col_names_map
+        super().__init__(self.name, self.name, self.content)
+
+    def format_message(self, rejected_rows: list[dict], rejected_rows_count: int, parsing_errors: list[dict]) -> str:
+        grouped_errors = {}
+
+        # Group errors by ROW_NUMBER
+        for i, row in enumerate(rejected_rows):
+            error_indices = {e["index"] for e in parsing_errors}
+            if i in error_indices:
+                continue
+            try:
+                row_number = row['ROW_NUMBER']
+                if row_number not in grouped_errors:
+                    grouped_errors[row_number] = {'rejected_record': row['REJECTED_RECORD'], 'errors': []}
+                grouped_errors[row_number]['errors'].append(f"Erroneous column: {row['COLUMN_NAME']}\nError message: {row['ERROR']}")
+            except Exception as e:
+                parsing_errors.append({"index": i, "message": str(e), "row": str(row)})
+
+        msg = f"Your data has been loaded but {rejected_rows_count} rows were skipped due to erroneous data. Here are the first {len(grouped_errors)} rejected rows:\n"
+        for row_number, data in grouped_errors.items():
+            msg += f"""Rejected record: {data['rejected_record']}"""
+            for error in data['errors']:
+                msg += f"- {error}\n"
+            msg += "\n"
+
+        return msg
+
+    """Replaces the autogenerated pyrel COLUMN_NAME in rejected_rows with the actual column names of the target table according to the mapping in col_names_map."""
+    def replace_column_names(self, rejected_rows: list[dict], col_names_map: dict) -> list[dict]:
+        # col_names_map looks like this: {col000: TIMESTAMPS, col002: id}
+        # COLUMN_NAME in rejected_rows looks like this: "OUT23A38988_10B1_4BCB_B566_89F7F3C8D1FB_INTERNAL"["COL000":1]
+
+        parsing_errors = []
+
+        # Convert col_names_map keys to lowercase for case-insensitive matching
+        col_names_map = {key.lower(): value for key, value in col_names_map.items()}
+        for i, row in enumerate(rejected_rows):
+            try:
+                col_name = row['COLUMN_NAME'].split('[')[1].split(':')[0].replace('"', '').strip().lower()
+                if col_name in col_names_map:
+                    row['COLUMN_NAME'] = col_names_map[col_name]
+            except Exception as e:
+                parsing_errors.append({"index": i, "message": str(e), "row": str(row)})
+        return parsing_errors
+    
 # --------------------------------------------------
 # ERP Exceptions
 # --------------------------------------------------
@@ -1349,18 +1406,19 @@ class EngineNotFoundException(RAIException):
         super().__init__(self.message, self.name, self.content)
 
     def format_message(self):
-        message = "We were unable to detect an existing engine or provision a new engine for you. You can manually create an engine"
+        message = "We were unable to detect an existing engine or provision a new engine for you. To manually create an engine:"
         return self.message_for_environment(
             message,
             f"rai engines:create --name {self.engine_name}",
             f"rai.Provider().create_engine('{self.engine_name}')"
         )
 
-class EngineAutoCreateFailed(RAIException):
-    def __init__(self, engine_name: str):
+class EngineProvisioningFailed(RAIException):
+    def __init__(self, engine_name: str, original_exception: Exception | None = None):
         self.engine_name = engine_name
-        self.message = "Engine auto-creation failed"
-        self.name = "Engine auto-creation failed"
+        self.original_exception = original_exception
+        self.message = "Engine provisioning failed"
+        self.name = self.message
 
         # Format the content and raw_content
         self.content = self.format_message()
@@ -1369,12 +1427,21 @@ class EngineAutoCreateFailed(RAIException):
         super().__init__(self.message, self.name, self.content)
 
     def format_message(self):
-        message = f"We tried to auto-create the engine [yellow]{self.engine_name}[/yellow] but it failed. You can manually create the engine"
-        return self.message_for_environment(
-            message,
-            f"rai engines:create --name {self.engine_name}",
-            f"rai.Provider().create_engine('{self.engine_name}')"
-        )
+        base_message = "\n".join([
+            f"We tried to provision the engine [yellow]{self.engine_name}[/yellow] but it failed."
+            "\n\n",
+            "To learn more about how to manually manage engines, see:"
+            "\n",
+            "https://docs.relational.ai/api/cli/engines"
+        ])
+
+        # Add original exception details if available
+        if self.original_exception:
+            original_msg = str(self.original_exception).strip()
+            if original_msg:
+                base_message += f"\n\n[red]Error:[/red] {original_msg}"
+
+        return base_message
 
 class DuoSecurityFailed(RAIException):
     def __init__(self, _: Exception):
@@ -1641,7 +1708,8 @@ class SnowflakeChangeTrackingNotEnabledException(RAIException):
         return (
             f"Change tracking isn't enabled for [yellow]{fqn_str}[/yellow].\n\n"
             f"To enable change tracking, you'll need to run the following SQL:\n\n"
-            f"{self.sql}"
+            f"{self.sql}\n\n"
+            f"If you want automatic enabling, please add [cyan]ensure_change_tracking = true[/cyan] to your config or Model"
         ).strip()
 
 class ModelNotFoundException(RAIException):
@@ -2083,43 +2151,77 @@ class SnowflakeProxySourceError(RAIException):
 # Direct Access Warnings
 #--------------------------------------------------
 
-class SessionResultFormatWarning(RAIWarning):
-    def __init__(self, use_direct_access: bool):
-        self.use_direct_access = use_direct_access
-        self.message = "Failed to alter the PYTHON_CONNECTOR_QUERY_RESULT_FORMAT session parameter."
-        self.name = "Session query result format mismatch"
+class DirectAccessInvalidAuthWarning(RAIWarning):
+    def __init__(self, authenticator, valid_authenticators):
+        self.authenticator = authenticator
+        self.valid_authenticators = valid_authenticators
+        self.name = "Direct Access Invalid Auth"
+        self.message = f"Due to security constraints in Snowflake, the selected authenticator '{authenticator}' is not supported when using direct access." 
+        self.content = self.format_message()
+        super().__init__(self.message, self.name, self.content)
 
-        # Format the content and raw_content
+    def format_message(self):
+        valid_authenticators_str = ", ".join(self.valid_authenticators)
+        return textwrap.dedent(f"""
+        {self.message}
+        Falling back to 'use_direct_access=False'. This comes with a potential negative latency impact.
+        Direct access requires one of the following authenticators: {valid_authenticators_str}
+        """)
+
+class InsecureKeychainWarning(RAIWarning):
+    def __init__(self):
+        self.message = "Insecure keyring detected. Please use a secure keyring backend."
+        self.name = "Insecure Keyring Warning"
         self.content = self.format_message()
-        self.raw_content = self.strip_rich_tags()
 
         super().__init__(self.message, self.name, self.content)
 
     def format_message(self):
-        if self.use_direct_access:
-            return textwrap.dedent(f"""
-            The PYTHON_CONNECTOR_QUERY_RESULT_FORMAT session parameter was not set to 'json'. Direct access will not work as expected.
-            Please ensure that the session is set to 'json' before using direct access.
-            If you provide the session make sure to construct with with the `json` result format:
-            [green] snowflake_session = Session.builder.configs({
-                "PYTHON_CONNECTOR_QUERY_RESULT_FORMAT": "json"
-                # other session parameters...
-            }).create() [/green]
-            """)
-        else:
-            return textwrap.dedent(f"""
-            The PYTHON_CONNECTOR_QUERY_RESULT_FORMAT session parameter was set to 'json' even though direct access is not used.
-            This might influence formatting of your results. To ensure default behavior you can construct your session like:
-            [green] snowflake_session = Session.builder.configs({
-                "PYTHON_CONNECTOR_QUERY_RESULT_FORMAT": "arrow"
-                # other session parameters...
-            }).create() [/green]
-            """)
+        return textwrap.dedent(f"""
+        {self.message}
+        
+        A secure keychain is used to cache refresh tokens for oauth authentication. Without
+        this caching mechanism a re-authentication will be required for each execution.
+        """)
+
+class KeychainFailureWarning(RAIWarning):
+    def __init__(self, message):
+        self.message = message
+        self.name = "Keychain Failure Warning"
+        self.content = self.format_message()
+
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent(f"""
+        {self.message}
+
+        This may be due to a misconfiguration or an issue with the keyring backend.
+        A secure keychain is used to cache refresh tokens for oauth authentication. Without
+        this caching mechanism a re-authentication will be required for each execution.
+        """)
 
 #--------------------------------------------------
 # Direct Access Exceptions
 #--------------------------------------------------
 
+class DirectAccessInvalidAuthException(RAIException):
+    def __init__(self, authenticator, valid_authenticators):
+        self.authenticator = authenticator
+        self.valid_authenticators = valid_authenticators
+        self.name = "Direct Access Invalid Auth"
+        self.message = f"Due to security constraints in Snowflake, the selected authenticator '{authenticator}' is not supported when using direct access." 
+        self.content = self.format_message()
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        valid_authenticators_str = ", ".join(self.valid_authenticators)
+        return textwrap.dedent(f"""
+        {self.message}
+        Use a valid authenticator or deactivate direct access via 'use_direct_access=False'.
+        Direct access requires one of the following authenticators: {valid_authenticators_str}
+        """)
+
 class ResponseStatusException(RAIException):
     def __init__(self, message: str, response):
         self.message = message
@@ -2139,6 +2241,26 @@ class ResponseStatusException(RAIException):
         except Exception:
             return self.response.text
 
+class OAuthFailedPortBinding(RAIException):
+    def __init__(self, redirect_port, exception):
+        self.redirect_port = redirect_port
+        self.message = f"Failed to bind to the OAuth redirect URI port {self.redirect_port}."
+        self.name = "OAuth Failed Port Binding Error"
+        self.exception = exception
+        self.content = self.format_message()
+        super().__init__(self.message, self.name, self.content)
+
+    def format_message(self):
+        return textwrap.dedent(f"""
+        {self.message}
+
+        This usually happens when the specified port for the OAuth redirect URI is not available.
+        You can fix the issue by ensuring that the port {self.redirect_port} is open and accessible.
+        You can check which processes are using this port with the command 'lsof -t -i :{self.redirect_port}' on Unix-based systems or 'netstat -ano | findstr :{self.redirect_port}' on Windows.
+        If you are sure no process should be running on this port, you can try killing any processes that are using it.
+
+        The original exception was: {self.exception}
+        """)
 
 #TODO: Take out call_source from Errors class and put it in the main module
 class Errors:
@@ -2183,3 +2305,93 @@ class LeftOverRelationException(RAIException):
 
         [green]rai imports:delete --object {self.import_name.upper()} --model {self.model} --force[/green]
         """)
+
+class UnsupportedColumnTypesWarning(RAIWarning):
+    def __init__(self, table_invalid_columns: dict[str, dict[str, str]]):
+        self.table_invalid_columns = table_invalid_columns
+        self.message = self._generate_message()
+        self.name = self._generate_name()
+        self.content = self.format_message()
+
+        super().__init__(self.message, self.name, self.content)
+
+    def _generate_message(self):
+        total_tables = len(self.table_invalid_columns)
+        total_columns = sum(len(invalid_columns) for invalid_columns in self.table_invalid_columns.values())
+
+        if total_tables == 1:
+            table_name = next(iter(self.table_invalid_columns.keys()))
+            return f"Found {total_columns} unsupported column type(s) in table '{table_name}'"
+        else:
+            return f"Found {total_columns} unsupported column type(s) across {total_tables} tables"
+
+    def _generate_name(self):
+        total_tables = len(self.table_invalid_columns)
+        total_columns = sum(len(invalid_columns) for invalid_columns in self.table_invalid_columns.values())
+
+        if total_tables == 1:
+            table_name = next(iter(self.table_invalid_columns.keys()))
+            return f"Unsupported Column Types - {total_columns} column(s) in {table_name}"
+        else:
+            return f"Unsupported Column Types - {total_columns} column(s) across {total_tables} tables"
+
+    def format_message(self):
+        def extract_type(data_type):
+            # Extract just the basic type from Snowflake data_type
+            # data_type might be a JSON string like {"type":"VARIANT","nullable":true}
+            # or a simple string like "VARIANT"
+            try:
+                parsed = json.loads(data_type)
+                if isinstance(parsed, dict) and "type" in parsed:
+                    return parsed["type"]
+                return str(parsed)
+            except (json.JSONDecodeError, TypeError):
+                # If it's not JSON, treat it as a simple string
+                if '(' in data_type:
+                    return data_type.split('(')[0]
+                return data_type
+
+        total_tables = len(self.table_invalid_columns)
+        total_columns = sum(len(invalid_columns) for invalid_columns in self.table_invalid_columns.values())
+
+        # Build organized display by table
+        table_sections = []
+        columns_displayed = 0
+        MAX_COLUMNS_TO_DISPLAY = 10
+
+        for table_name, invalid_columns in self.table_invalid_columns.items():
+            table_columns = []
+            for col_name, data_type in invalid_columns.items():
+                if columns_displayed >= MAX_COLUMNS_TO_DISPLAY:
+                    break
+                type_only = extract_type(data_type)
+                table_columns.append(f"  [yellow]{col_name}[/yellow]: [red]{type_only}[/red]")
+                columns_displayed += 1
+
+            if table_columns:
+                table_sections.append(f"[yellow]{table_name}[/yellow]:\n" + "\n".join(table_columns))
+
+            if columns_displayed >= MAX_COLUMNS_TO_DISPLAY:
+                break
+
+        # Build the message
+        if total_tables == 1:
+            table_name = next(iter(self.table_invalid_columns.keys()))
+            message = f"Found {total_columns} unsupported column type(s) in table '{table_name}':"
+        else:
+            message = f"Found {total_columns} unsupported column type(s) across {total_tables} tables:"
+
+        if total_columns > MAX_COLUMNS_TO_DISPLAY:
+            message += f" (showing first {MAX_COLUMNS_TO_DISPLAY} for brevity)"
+
+        note = "These columns will not be accessible in your model.\n\nFor the list of supported column types see: https://docs.relational.ai/api/cli/imports/stream/#supported-column-types"
+
+        columns_text = "\n\n".join(table_sections)
+
+        return textwrap.dedent(f"""
+{message}
+
+{columns_text}
+
+{note}
+""")
