@@ -64,14 +64,6 @@ class SolverModel:
                     {scope}serialized[h], _solver_unwrap[{scope}variable_name], s
                 )
 
-            def {scope}specialized_components(t, h, s):
-                exists((v) | {{
-                    (:min_objective, {scope}min_objectives);
-                    (:max_objective, {scope}max_objectives);
-                    (:constraint, {scope}constraints);
-                    }}(t, v) and pyrel_unwrap(v, h) and {scope}serialized(v, s)
-                )
-
             declare {scope}solve_output
         """))
         return None
@@ -139,13 +131,7 @@ class SolverModel:
         return self._add_component(self.MaxObjective, expr, name_args)
 
     def _add_component(self, typ, expr, name_args:List|None):
-        # TODO(coey) for now we create unique property names for the serialized component
-        # property in order to get unique relation names in the generated Rel, otherwise
-        # we have trouble tracing performance data back to the original PyRel block.
-        prop = "serialized" + str(dsl.next_id())
-        comp = typ.add(**{prop: _wrap_expr(expr)})
-        self.graph.install_raw(f"def {self.scope}serialized {{ {self.scope}{prop} }}")
-        # comp = typ.add(serialized=_wrap_expr(expr)) # TODO(coey) the simpler way.
+        comp = typ.add(serialized=_wrap_expr(expr))
         if name_args:
             comp.set(component_name=make_string(name_args))
         return comp
@@ -223,7 +209,14 @@ class SolverModel:
         self.graph.exec_raw("")
         response = self.graph.exec_raw(
             textwrap.dedent(f"""
-            ic model_not_empty("Solver model is empty.") requires not empty({scope}model_string)
+            @inline
+            def {scope}specialized_components(t, h, s):
+                exists((v) | {{
+                    (:min_objective, {scope}min_objectives);
+                    (:max_objective, {scope}max_objectives);
+                    (:constraint, {scope}constraints);
+                    }}(t, v) and pyrel_unwrap(v, h) and {scope}serialized(v, s)
+                )
 
             @no_diagnostics(:EXPERIMENTAL)
             def {scope}model_string {{
@@ -236,6 +229,8 @@ class SolverModel:
                 }}]
             }}
 
+            ic model_not_empty("Solver model is empty.") requires not empty({scope}model_string)
+
             def config[:envelope, :content_type]: "application/octet-stream"
             def config[:envelope, :payload, :data]: {scope}model_string
             def config[:envelope, :payload, :path]: "{model_uri}"
