@@ -1075,6 +1075,72 @@ def engines_suspend(name):
             exit_with_divider(1)
     divider()
 
+#--------------------------------------------------
+# Engine alter engine pool
+#--------------------------------------------------
+
+@cli.command(name="engines:alter_pool", help="Alter the engine pool size")
+@click.option("--size", help="Engine size")
+@click.option("--min", help="Minimum number of engines")
+@click.option("--max", help="Maximum number of engines")
+def engines_alter_pool(size:str|None=None, min:int|None=None, max:int|None=None):
+    divider(flush=True)
+    ensure_config()
+    provider = get_resource_provider()
+
+    if provider.platform != "snowflake":
+        exit_with_error("Engine pool alteration is only supported for Snowflake")
+
+    # Ask for engine size if not provided
+    if not size:
+        valid_sizes = provider.get_engine_sizes()
+        size = controls.fuzzy(
+            "Select engine size:",
+            choices=valid_sizes,
+            mandatory=True
+        )
+
+    # Validate engine size
+    valid_sizes = provider.get_engine_sizes()
+    if size not in valid_sizes:
+        exit_with_error(f"Invalid engine size '{size}'. Valid sizes: {valid_sizes}")
+
+    # Ask for minimum number of engines
+    if min is None:
+        min_str = controls.text(
+            "Enter minimum number of engines:",
+            validator=lambda x: x.isdigit() and int(x) > 0,
+            invalid_message="Please enter a valid non-negative integer"
+        )
+        min = int(min_str)
+    else:
+        # Convert string to int if it came from command line
+        min = int(min)
+
+    # Ask for maximum number of engines
+    if max is None:
+        max_str = controls.text(
+            "Enter maximum number of engines:",
+            validator=lambda x: x.isdigit() and int(x) >= min,
+            invalid_message=f"Please enter a valid integer greater than or equal to {min}"
+        )
+        max = int(max_str)
+    else:
+        # Convert string to int if it came from command line
+        max = int(max)
+
+    # Validate that range is valid
+    if max < min:
+        exit_with_error(f"Maximum number of engines ({max}) must be greater than or equal to minimum ({min})")
+
+    rich.print()
+
+    # Call the API method
+    with Spinner("Altering engine pool", "Engine pool altered", "Error:"):
+        # Type cast to ensure type checker recognizes the method
+        cast(ResourcesBase, provider).alter_engine_pool(size, min, max)
+    divider()
+
 #--------------------------------------------------
 # Import Source flows
 #--------------------------------------------------
