@@ -8,6 +8,7 @@ USE_DIRECT_ACCESS = False
 WAIT_FOR_STREAM_SYNC = True
 USE_PACKAGE_MANAGER = True
 SHOW_FULL_TRACES = False
+SHOW_DEBUG_LOGS = False
 
 #--------------------------------------------------
 # Constants
@@ -51,12 +52,39 @@ SNOWFLAKE_PROFILE_DEFAULTS = {
     "use_direct_access": { "value": USE_DIRECT_ACCESS, "required": False },
 }
 
+SNOWFLAKE_DIRECT_ACCESS_AUTHS = {
+    "oauth_authorization_code": {
+        "authenticator": { "value": "oauth", "required": True },
+        "oauth_client_id": { "value": FIELD_PLACEHOLDER, "required": True },
+        # client secret not mandatory, only needed for non-public security integrations
+        "oauth_client_secret": { "value": FIELD_PLACEHOLDER, "required": False },
+        "oauth_redirect_uri": { "value": FIELD_PLACEHOLDER, "required": True },
+        **SNOWFLAKE_PROFILE_DEFAULTS,
+    },
+    "programmatic_access_token": {
+        "authenticator": { "value": "programmatic_access_token", "required": True },
+        "token_file_path": { "value": FIELD_PLACEHOLDER, "required": True },
+        **SNOWFLAKE_PROFILE_DEFAULTS,
+    },
+    "snowflake_jwt": {
+        "authenticator": { "value": "snowflake_jwt", "required": True },
+        "private_key_file": { "value": FIELD_PLACEHOLDER, "required": True },
+        "private_key_file_pwd": { "value": FIELD_PLACEHOLDER, "required": False },
+        **SNOWFLAKE_PROFILE_DEFAULTS,
+    },
+}
+
 SNOWFLAKE_AUTHS = {
     "snowflake": {
         "authenticator": { "value": "snowflake", "required": True },
         "password": { "value": FIELD_PLACEHOLDER, "required": True },
         **SNOWFLAKE_PROFILE_DEFAULTS,
     },
+    "snowflake_jwt": {
+        "authenticator": { "value": "snowflake_jwt", "required": True },
+        "private_key_file": { "value": FIELD_PLACEHOLDER, "required": True },
+        **SNOWFLAKE_PROFILE_DEFAULTS,
+    },
     "username_password_mfa": {
         "authenticator": { "value": "username_password_mfa", "required": True },
         "password": { "value": FIELD_PLACEHOLDER, "required": True },
@@ -67,6 +95,7 @@ SNOWFLAKE_AUTHS = {
         "authenticator": { "value": "externalbrowser", "required": True },
         **SNOWFLAKE_PROFILE_DEFAULTS,
     },
+    **SNOWFLAKE_DIRECT_ACCESS_AUTHS,
 }
 
 AZURE_ENVS = {
