@@ -433,8 +433,8 @@ def run_unit_tests(test_path, update, verbose):
         test_path = "tests/early_access/unit/ir"
     elif test_path == "rel":
         test_path = "tests/early_access/unit/rel"
-    elif test_path == "devmode":
-        test_path = "tests/early_access/unit/devmode"
+    elif test_path == "sql":
+        test_path = "tests/early_access/unit/sql"
 
     # Make sure the test_path is an absolute path
     if not os.path.isabs(test_path):
