@@ -197,7 +197,7 @@ def span_code(span:SpanNode):
         with ui.column().style("gap:0px; padding-left: 30px; "):
             if toggles["dsl"]:
                 code_with_header("dsl", span.start_attrs["dsl"])
-            if toggles["metamodel"]:
+            if toggles["metamodel"] and "metamodel" in span.end_attrs:
                 code_with_header("metamodel", span.end_attrs["metamodel"])
     elif span.type == "query":
         if span.start_attrs.get("source"):
@@ -217,7 +217,7 @@ def span_code(span:SpanNode):
         if toggles["rel"]:
             code_with_header("rel model", span.end_attrs.get("rel", '// No Rel was emitted.'), language="rel")
         if toggles["lqp"]:
-            code_with_header("lqp model", span.end_attrs.get("lqp_debug", ';; No LQP was emitted.'), language="lqp")
+            code_with_header("lqp model", span.end_attrs.get("lqp", ';; No LQP was emitted.'), language="lqp")
         if toggles["sql"]:
             code_with_header("sql model", span.end_attrs.get("sql", '-- No SQL was emitted.'), language="sql")
     elif span.type == "compile" and span.end_attrs.get("compile_type") == "query":
@@ -228,7 +228,7 @@ def span_code(span:SpanNode):
         if toggles["rel"]:
             code_with_header("rel query", span.end_attrs.get("rel", '// No Rel was emitted.'), language="rel")
         if toggles["lqp"]:
-            code_with_header("lqp query", span.end_attrs.get("lqp_debug", ';; No LQP was emitted.'), language="lqp")
+            code_with_header("lqp query", span.end_attrs.get("lqp", ';; No LQP was emitted.'), language="lqp")
         if toggles["sql"]:
             code_with_header("sql query", span.end_attrs.get("sql", '-- No SQL was emitted.'), language="sql")
     elif span.type == "passes":
@@ -236,11 +236,13 @@ def span_code(span:SpanNode):
         if toggles["passes"]:
             previous_code = None
             for child in span.children:
-                current_code = child.end_attrs["metamodel"]
-                if current_code == previous_code:
-                    current_code = ""
-                else:
-                    previous_code = current_code
+                current_code = ""
+                if "metamodel" in child.end_attrs:
+                    current_code = child.end_attrs["metamodel"]
+                    if current_code == previous_code:
+                        current_code = ""
+                    else:
+                        previous_code = current_code
                 code_element = code_with_header(child.type + f' ({format_time(child.elapsed)})', current_code)
                 code_element.props(f'id="pass-{child.id}"')
                 code_element.style("scroll-margin-top: 40px;")
