@@ -52,6 +52,8 @@ def engine_config_fixture(size, use_direct_access=False, reset_session=False, ge
     # the engine specified there.
     config = cfg.Config()
     if config.file_path is not None:
+        # Set test defaults
+        config.set("reuse_model", False)
         # Try to reset the session instead of using active session if reset_session is true
         rai.Resources(config=config, reset_session=reset_session, generation=generation)
         yield config
@@ -82,9 +84,23 @@ def delete_engine(engine_name: str, use_direct_access=False):
     provider.delete_engine(engine_name)
     print(f"Engine {engine_name} deleted")
 
-def make_config(engine_name: str | None = None, fetch_profile: bool = True, use_package_manager = USE_PACKAGE_MANAGER, use_direct_access = False, show_full_traces = True) -> cfg.Config:
+def make_config(engine_name: str | None = None, fetch_profile: bool = True, use_package_manager = USE_PACKAGE_MANAGER, use_direct_access = False, show_full_traces = True, show_debug_logs = True, reuse_model = False) -> cfg.Config:
+    # First try to load from raiconfig.toml
+    try:
+        config = cfg.Config()
+        if config.file_path is not None:
+            # Set test defaults
+            config.set("reuse_model", reuse_model)
+            rai.Resources(config=config)
+            return config
+    except Exception as e:
+        print(f"Could not load config from file: {e}, trying environment variables")
+    # If that fails, construct from environment variables
     cloud_provider = os.getenv("RAI_CLOUD_PROVIDER")
 
+    if cloud_provider is None:
+        cloud_provider = os.getenv("RAI_CLOUD_PROVIDER")
+
     print('cloud provider:', cloud_provider)
 
     if cloud_provider is None:
@@ -116,6 +132,8 @@ def make_config(engine_name: str | None = None, fetch_profile: bool = True, use_
                 "engine": engine_name,
                 "use_package_manager": use_package_manager,
                 "show_full_traces": show_full_traces,
+                "show_debug_logs": show_debug_logs,
+                "reuse_model": reuse_model,
             }
         )
 
@@ -142,9 +160,22 @@ def make_config(engine_name: str | None = None, fetch_profile: bool = True, use_
             "use_package_manager": use_package_manager,
             "use_direct_access": use_direct_access,
             "show_full_traces": show_full_traces,
+            "show_debug_logs": show_debug_logs,
+            "reuse_model": reuse_model,
         }
         if engine_name:
             current_config["engine"] = engine_name
+
+        # For direct access, we use key-pair as the default for testing and need to configure additional parameters.
+        if use_direct_access:
+            authenticator=os.getenv("AUTHENTICATOR","")
+            private_key_file=os.getenv("PRIVATE_KEY_FILE","")
+            private_key_file_pwd=os.getenv("SF_TEST_ACCOUNT_KEY_PASSPHRASE","")
+
+            current_config["authenticator"] = authenticator
+            current_config["private_key_file"] = private_key_file
+            current_config["private_key_file_pwd"]=private_key_file_pwd
+
         return cfg.Config(current_config, fetch=fetch_profile)
 
     else:
